@@ -123,11 +123,11 @@ get_form_mode(struct html_context *html_context, unsigned char *attr)
 	return FORM_MODE_NORMAL;
 }
 
-static struct el_form_control *
+static struct form_control *
 init_form_control(enum form_type type, unsigned char *attr,
                   struct html_context *html_context)
 {
-	struct el_form_control *fc;
+	struct form_control *fc;
 
 	fc = mem_calloc(1, sizeof(*fc));
 	if (!fc) return NULL;
@@ -144,7 +144,7 @@ html_button(struct html_context *html_context, unsigned char *a,
             unsigned char *xxx3, unsigned char *xxx4, unsigned char **xxx5)
 {
 	unsigned char *al;
-	struct el_form_control *fc;
+	struct form_control *fc;
 	enum form_type type = FC_SUBMIT;
 	int cp = html_context->doc_cp;
 
@@ -189,7 +189,7 @@ no_type_attr:
 
 static void
 html_input_format(struct html_context *html_context, unsigned char *a,
-	   	  struct el_form_control *fc)
+	   	  struct form_control *fc)
 {
 	put_chrs(html_context, " ", 1);
 	html_stack_dup(html_context, ELEMENT_KILLABLE);
@@ -270,7 +270,7 @@ html_input(struct html_context *html_context, unsigned char *a,
            unsigned char *xxx3, unsigned char *xxx4, unsigned char **xxx5)
 {
 	unsigned char *al;
-	struct el_form_control *fc;
+	struct form_control *fc;
 	int cp = html_context->doc_cp;
 
 	fc = init_form_control(FC_TEXT, a, html_context);
@@ -316,8 +316,8 @@ html_input(struct html_context *html_context, unsigned char *a,
 	if (fc->size == -1)
 		fc->size = html_context->options->default_form_input_size;
 	fc->size++;
-	if (fc->size > html_context->options->document_width)
-		fc->size = html_context->options->document_width;
+	if (fc->size > html_context->options->box.width)
+		fc->size = html_context->options->box.width;
 	fc->maxlength = get_num(a, "maxlength", cp);
 	if (fc->maxlength == -1) fc->maxlength = INT_MAX;
 	if (fc->type == FC_CHECKBOX || fc->type == FC_RADIO)
@@ -340,7 +340,7 @@ do_html_select(unsigned char *attr, unsigned char *html,
 	       struct html_context *html_context)
 {
 	struct conv_table *ct = html_context->special_f(html_context, SP_TABLE, NULL);
-	struct el_form_control *fc;
+	struct form_control *fc;
 	struct string lbl = NULL_STRING, orig_lbl = NULL_STRING;
 	unsigned char **values = NULL;
 	unsigned char **labels;
@@ -494,10 +494,10 @@ end_parse:
 	fc->labels = labels;
 
 	menu_labels(fc->menu, "", labels);
+	put_chrs(html_context, "[", 1);
 	html_stack_dup(html_context, ELEMENT_KILLABLE);
 	format.form = fc;
 	format.style.attr |= AT_BOLD;
-	put_chrs(html_context, "[&nbsp;", 7);
 
 	max_width = 0;
 	for (i = 0; i < order; i++) {
@@ -514,8 +514,8 @@ end_parse:
 	for (i = 0; i < max_width; i++)
 		put_chrs(html_context, "_", 1);
 
-	put_chrs(html_context, "&nbsp;]", 7);
 	pop_html_element(html_context);
+	put_chrs(html_context, "]", 1);
 	html_context->special_f(html_context, SP_CONTROL, fc);
 }
 
@@ -551,7 +551,7 @@ void
 html_option(struct html_context *html_context, unsigned char *a,
             unsigned char *xxx3, unsigned char *xxx4, unsigned char **xxx5)
 {
-	struct el_form_control *fc;
+	struct form_control *fc;
 	unsigned char *val;
 
 	if (!format.select) return;
@@ -629,7 +629,7 @@ void
 html_textarea(struct html_context *html_context, unsigned char *attr,
               unsigned char *html, unsigned char *eof, unsigned char **end)
 {
-	struct el_form_control *fc;
+	struct form_control *fc;
 	unsigned char *p, *t_name, *wrap_attr;
 	int t_namelen;
 	int cols, rows;
@@ -678,8 +678,8 @@ pp:
 		cols = html_context->options->default_form_input_size;
 	cols++; /* Add 1 column, other browsers may have different
 		   behavior here (mozilla adds 2) --Zas */
-	if (cols > html_context->options->document_width)
-		cols = html_context->options->document_width;
+	if (cols > html_context->options->box.width)
+		cols = html_context->options->box.width;
 	fc->cols = cols;
 
 	rows = get_num(attr, "rows", html_context->doc_cp);
