@@ -63,6 +63,7 @@
 
 #elif defined(CONFIG_GNUTLS)
 
+#define ssl_do_connect(conn)		gnutls_handshake(*((ssl_t *) socket->ssl))
 #define ssl_do_write(socket, data, len)	gnutls_record_send(*((ssl_t *) socket->ssl), data, len)
 #define ssl_do_read(socket, data, len)	gnutls_record_recv(*((ssl_t *) socket->ssl), data, len)
 /* We probably don't handle this entirely correctly.. */
@@ -80,9 +81,7 @@ static void
 ssl_set_no_tls(struct socket *socket)
 {
 #ifdef CONFIG_OPENSSL
-#ifdef SSL_OP_NO_TLSv1
-	SSL_set_options((ssl_t *)socket->ssl, SSL_OP_NO_TLSv1);
-#endif
+	((ssl_t *) socket->ssl)->options |= SSL_OP_NO_TLSv1;
 #elif defined(CONFIG_GNUTLS)
 	/* There is another gnutls_priority_set_direct call elsewhere
 	 * in ELinks.  If you change the priorities here, please check
@@ -364,22 +363,6 @@ verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 
 #endif	/* USE_OPENSSL */
 
-#if defined(CONFIG_GNUTLS)
-static int
-ssl_do_connect(struct socket *socket)
-{
-	int ret;
-
-	gnutls_handshake_set_timeout(*(ssl_t *)(socket->ssl), GNUTLS_DEFAULT_HANDSHAKE_TIMEOUT);
-
-	do {
-		ret = gnutls_handshake(*(ssl_t *)socket->ssl);
-	} while (ret < 0 && !gnutls_error_is_fatal(ret));
-
-	return ret;
-}
-#endif
-
 static void
 ssl_want_read(struct socket *socket)
 {
@@ -389,7 +372,7 @@ ssl_want_read(struct socket *socket)
 	switch (ssl_do_connect(socket)) {
 		case SSL_ERROR_NONE:
 #ifdef CONFIG_GNUTLS
-			if (socket->verify && get_opt_bool("connection.ssl.cert_verify", NULL)
+			if (get_opt_bool("connection.ssl.cert_verify", NULL)
 			    && verify_certificates(socket)) {
 				socket->ops->retry(socket, connection_state(S_SSL_ERROR));
 				return;
@@ -420,7 +403,6 @@ ssl_connect(struct socket *socket)
 
 	/* TODO: Recode server_name to UTF-8.  */
 	server_name = get_uri_string(conn->proxied_uri, URI_HOST);
-
 	if (!server_name) {
 		socket->ops->done(socket, connection_state(S_OUT_OF_MEM));
 		return -1;
@@ -444,7 +426,7 @@ ssl_connect(struct socket *socket)
 #ifdef USE_OPENSSL
 	SSL_set_fd(socket->ssl, socket->fd);
 
-	if (socket->verify && get_opt_bool("connection.ssl.cert_verify", NULL))
+	if (get_opt_bool("connection.ssl.cert_verify", NULL))
 		SSL_set_verify(socket->ssl, SSL_VERIFY_PEER
 					  | SSL_VERIFY_FAIL_IF_NO_PEER_CERT,
 			       verify_callback);
@@ -471,7 +453,7 @@ ssl_connect(struct socket *socket)
 					(SSL *) socket->ssl,
 					client_cert);
 #else
-			SSL_CTX *ctx = SSL_get_SSL_CTX((SSL *) socket->ssl);
+			SSL_CTX *ctx = ((SSL *) socket->ssl)->ctx;
 
 			SSL_CTX_use_certificate_chain_file(ctx, client_cert);
 			SSL_CTX_use_PrivateKey_file(ctx, client_cert,
@@ -506,7 +488,7 @@ ssl_connect(struct socket *socket)
 
 		case SSL_ERROR_NONE:
 #ifdef CONFIG_GNUTLS
-			if (!socket->verify || !get_opt_bool("connection.ssl.cert_verify", NULL))
+			if (!get_opt_bool("connection.ssl.cert_verify", NULL))
 				break;
 
 			if (!verify_certificates(socket))
@@ -518,6 +500,7 @@ ssl_connect(struct socket *socket)
 				/* DBG("sslerr %s", gnutls_strerror(ret)); */
 				socket->no_tls = !socket->no_tls;
 			}
+
 			connect_socket(socket, connection_state(S_SSL_ERROR));
 			return -1;
 	}
@@ -568,16 +551,8 @@ ssl_read(struct socket *socket, unsigned char *data, int len)
 #endif
 
 #ifdef CONFIG_GNUTLS
-		if (err == GNUTLS_E_PREMATURE_TERMINATION) {
-			return SOCKET_CANT_READ;
-		}
-
-		if (err == GNUTLS_E_REHANDSHAKE) {
-			err = ssl_do_connect(socket);
-			if (err == 0) {
-				return SOCKET_SSL_WANT_READ;
-			}
-		}
+		if (err == GNUTLS_E_REHANDSHAKE)
+			return -1;
 #endif
 
 		if (err == SSL_ERROR_WANT_READ ||
