@@ -21,9 +21,6 @@
 #include "terminal/kbd.h"
 #include "terminal/screen.h"
 #include "terminal/terminal.h"
-#ifdef CONFIG_TERMINFO
-#include "terminal/terminfo.h"
-#endif
 #include "util/conv.h"
 #include "util/error.h"
 #include "util/memory.h"
@@ -249,11 +246,6 @@ struct screen_driver_opt {
 	/* Whether the terminal supports combining characters. */
 	unsigned int combine:1;
 #endif /* CONFIG_COMBINE */
-
-#ifdef CONFIG_TERMINFO
-	/* Whether use terminfo. */
-	unsigned int terminfo:1;
-#endif
 };
 
 /** Used in @c add_char*() and @c redraw_screen() to reduce the logic.
@@ -293,9 +285,6 @@ static const struct screen_driver_opt dumb_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
-#ifdef CONFIG_TERMINFO
-	/* terminfo */		0,
-#endif
 };
 
 /** Default options for ::TERM_VT100.  */
@@ -316,9 +305,6 @@ static const struct screen_driver_opt vt100_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
-#ifdef CONFIG_TERMINFO
-	/* terminfo */		0,
-#endif
 };
 
 /** Default options for ::TERM_LINUX.  */
@@ -339,9 +325,6 @@ static const struct screen_driver_opt linux_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
-#ifdef CONFIG_TERMINFO
-	/* terminfo */		0,
-#endif
 };
 
 /** Default options for ::TERM_KOI8.  */
@@ -362,9 +345,6 @@ static const struct screen_driver_opt koi8_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
-#ifdef CONFIG_TERMINFO
-	/* terminfo */		0,
-#endif
 };
 
 /** Default options for ::TERM_FREEBSD.  */
@@ -385,9 +365,6 @@ static const struct screen_driver_opt freebsd_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
-#ifdef CONFIG_TERMINFO
-	/* terminfo */		0,
-#endif
 };
 
 /** Default options for ::TERM_FBTERM.  */
@@ -408,9 +385,6 @@ static const struct screen_driver_opt fbterm_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
-#ifdef CONFIG_TERMINFO
-	/* terminfo */		0,
-#endif
 };
 
 /** Default options for all the different types of terminals.
@@ -527,49 +501,7 @@ set_screen_driver_opt(struct screen_driver *driver, struct option *term_spec)
 		} else if (driver->type == TERM_VT100) {
 			driver->opt.frame = frame_vt100;
 		}
-	} /* !utf8_io */
-#ifdef CONFIG_TERMINFO
-	driver->opt.terminfo = get_cmd_opt_bool("terminfo");
-	if (!driver->opt.terminfo) {
-		return;
-	} 
-#ifdef CONFIG_TRUE_COLOR
-	if (driver->opt.color_mode == COLOR_MODE_TRUE_COLOR) {
-		driver->opt.terminfo = 0;
-		return;
-	}
-#endif
-	if (driver->opt.color_mode == COLOR_MODE_MONO) {
-		driver->opt.terminfo = 0;
-		return;
-	}
-
-	switch (terminfo_max_colors()) {
-	case 88:
-#ifdef CONFIG_88_COLORS
-		driver->opt.color_mode = COLOR_MODE_88;
-#else
-		driver->opt.color_mode = COLOR_MODE_16;
-#endif
-		break;
-
-	case 256:
-#ifdef CONFIG_256_COLORS
-		driver->opt.color_mode = COLOR_MODE_256;
-#else
-		driver->opt.color_mode = COLOR_MODE_16;
-#endif
-		break;
-
-	case 16:
-	case 8:
-		driver->opt.color_mode = COLOR_MODE_16;
-		break;
-	default:
-		driver->opt.color_mode = COLOR_MODE_MONO;
-		break;
-	}
-#endif
+ 	} /* !utf8_io */
 }
 
 static int
@@ -652,13 +584,8 @@ done_screen_drivers(struct module *xxx)
 static inline struct string *
 add_cursor_move_to_string(struct string *screen, int y, int x)
 {
-#ifdef CONFIG_TERMINFO
-	if (get_cmd_opt_bool("terminfo")) {
-		return add_to_string(screen, terminfo_cursor_address(y-1, x-1));
-	} else
-#endif
 #define CURSOR_NUM_LEN 10 /* 10 chars for @y and @x numbers should be more than enough. */
-{	unsigned char code[4 + 2 * CURSOR_NUM_LEN + 1];
+	unsigned char code[4 + 2 * CURSOR_NUM_LEN + 1];
 	unsigned int length = 2;
 
 	code[0] = '\033';
@@ -675,7 +602,6 @@ add_cursor_move_to_string(struct string *screen, int y, int x)
 	code[length++] = 'H';
 
 	return add_bytes_to_string(screen, code, length);
-}
 #undef CURSOR_NUM_LEN
 }
 
@@ -880,14 +806,7 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	    italic != state->italic && driver->opt.italic
 	   ) {
 		state->italic = italic;
-#ifdef CONFIG_TERMINFO
-		if (driver->opt.terminfo) {
-			add_to_string(screen, terminfo_set_italics(italic));
-		} else
-#endif
-		{
-			add_term_string(screen, driver->opt.italic[!!italic]);
-		}
+		add_term_string(screen, driver->opt.italic[!!italic]);
 	}
 
 	if (
@@ -897,14 +816,7 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	    underline != state->underline && driver->opt.underline
 	   ) {
 		state->underline = underline;
-#ifdef CONFIG_TERMINFO
-		if (driver->opt.terminfo) {
-			add_to_string(screen, terminfo_set_underline(underline));
-		} else
-#endif
-		{
-			add_term_string(screen, driver->opt.underline[!!underline]);
-		}
+		add_term_string(screen, driver->opt.underline[!!underline]);
 	}
 
 	if (
@@ -915,14 +827,7 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	   ) {
 		state->bold = bold;
 		if (bold) {
-#ifdef CONFIG_TERMINFO
-			if (driver->opt.terminfo) {
-				add_to_string(screen, terminfo_set_bold(bold));
-			} else
-#endif
-			{
-				add_bytes_to_string(screen, "\033[1m", 4);
-			}
+			add_bytes_to_string(screen, "\033[1m", 4);
 		} else {
 			/* Force repainting of the other attributes. */
 			state->color[0] = ch->c.color[0] + 1;
@@ -937,65 +842,49 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	   ) {
 		copy_color_16(state->color, ch->c.color);
 
-#ifdef CONFIG_TERMINFO
-		if (driver->opt.terminfo) {
-			add_to_string(screen, terminfo_set_bold(bold));
-			add_to_string(screen, terminfo_set_foreground(TERM_COLOR_FOREGROUND_16(ch->c.color)));
-			add_to_string(screen, terminfo_set_background(TERM_COLOR_BACKGROUND_16(ch->c.color)));
+		add_bytes_to_string(screen, "\033[0", 3);
 
-			if (italic)
-				add_to_string(screen, terminfo_set_italics(italic));
+		/* @set_screen_driver_opt has set @driver->opt.color_mode
+		 * according to terminal-type-specific options.
+		 * The caller of @add_char16 has already partially
+		 * checked it, but there are still these possibilities:
+		 * - COLOR_MODE_MONO.  Then don't show colors, but
+		 *   perhaps use the standout attribute.
+		 * - COLOR_MODE_16.  Use 16 colors.
+		 * - An unsupported color mode.  Use 16 colors.  */
+		if (driver->opt.color_mode != COLOR_MODE_MONO) {
+			unsigned char code[6] = ";30;40";
+			unsigned char bgcolor = TERM_COLOR_BACKGROUND_16(ch->c.color);
 
-			if (underline)
-				add_to_string(screen, terminfo_set_underline(underline));
+			code[2] += TERM_COLOR_FOREGROUND_16(ch->c.color);
 
-		} else
-#endif
-		{
-			add_bytes_to_string(screen, "\033[0", 3);
-
-			/* @set_screen_driver_opt has set @driver->opt.color_mode
-			 * according to terminal-type-specific options.
-			 * The caller of @add_char16 has already partially
-			 * checked it, but there are still these possibilities:
-			 * - COLOR_MODE_MONO.  Then don't show colors, but
-			 *   perhaps use the standout attribute.
-			 * - COLOR_MODE_16.  Use 16 colors.
-			 * - An unsupported color mode.  Use 16 colors.  */
-			if (driver->opt.color_mode != COLOR_MODE_MONO) {
-				unsigned char code[6] = ";30;40";
-				unsigned char bgcolor = TERM_COLOR_BACKGROUND_16(ch->c.color);
-
-				code[2] += TERM_COLOR_FOREGROUND_16(ch->c.color);
-
-				if (!driver->opt.transparent || bgcolor != 0) {
-					code[5] += bgcolor;
-					add_bytes_to_string(screen, code, 6);
-				} else {
-					add_bytes_to_string(screen, code, 3);
-				}
-
-			} else if (ch->attr & SCREEN_ATTR_STANDOUT) {
-				/* Flip the fore- and background colors for highlighing
-				 * purposes. */
-				add_bytes_to_string(screen, ";7", 2);
+			if (!driver->opt.transparent || bgcolor != 0) {
+				code[5] += bgcolor;
+				add_bytes_to_string(screen, code, 6);
+			} else {
+				add_bytes_to_string(screen, code, 3);
 			}
 
-			if (italic && driver->opt.italic) {
-				add_bytes_to_string(screen, ";3", 2);
-			}
-
-			if (underline && driver->opt.underline) {
-				add_bytes_to_string(screen, ";4", 2);
-			}
-
-			/* Check if the char should be rendered bold. */
-			if (bold) {
-				add_bytes_to_string(screen, ";1", 2);
-			}
-
-			add_bytes_to_string(screen, "m", 1);
+		} else if (ch->attr & SCREEN_ATTR_STANDOUT) {
+			/* Flip the fore- and background colors for highlighing
+			 * purposes. */
+			add_bytes_to_string(screen, ";7", 2);
 		}
+
+		if (italic && driver->opt.italic) {
+			add_bytes_to_string(screen, ";3", 2);
+		}
+
+		if (underline && driver->opt.underline) {
+			add_bytes_to_string(screen, ";4", 2);
+		}
+
+		/* Check if the char should be rendered bold. */
+		if (bold) {
+			add_bytes_to_string(screen, ";1", 2);
+		}
+
+		add_bytes_to_string(screen, "m", 1);
 	}
 
 	add_char_data(screen, driver, ch->data, border);
@@ -1072,38 +961,17 @@ add_char256(struct string *screen, struct screen_driver *driver,
 
 		if ((attr_delta & SCREEN_ATTR_ITALIC) && driver->opt.italic) {
 			state->italic = !!(ch->attr & SCREEN_ATTR_ITALIC);
-#ifdef CONFIG_TERMINFO
-			if (driver->opt.terminfo) {
-				add_to_string(screen, terminfo_set_italics(state->italic));
-			} else
-#endif
-			{
-				add_term_string(screen, driver->opt.italic[state->italic]);
-			}
+			add_term_string(screen, driver->opt.italic[state->italic]);
 		}
 
 		if ((attr_delta & SCREEN_ATTR_UNDERLINE) && driver->opt.underline) {
 			state->underline = !!(ch->attr & SCREEN_ATTR_UNDERLINE);
-#ifdef CONFIG_TERMINFO
-			if (driver->opt.terminfo) {
-				add_to_string(screen, terminfo_set_underline(state->underline));
-			} else
-#endif
-			{
-				add_term_string(screen, driver->opt.underline[state->underline]);
-			}
+			add_term_string(screen, driver->opt.underline[state->underline]);
 		}
 
 		if (attr_delta & SCREEN_ATTR_BOLD) {
 			if (ch->attr & SCREEN_ATTR_BOLD) {
-#ifdef CONFIG_TERMINFO
-				if (driver->opt.terminfo) {
-					add_to_string(screen, terminfo_set_bold(1));
-				} else
-#endif
-				{
-					add_bytes_to_string(screen, "\033[1m", 4);
-				}
+				add_bytes_to_string(screen, "\033[1m", 4);
 			} else {
 				/* Force repainting of the other attributes. */
 				state->color[0] = ch->c.color[0] + 1;
@@ -1120,41 +988,23 @@ add_char256(struct string *screen, struct screen_driver *driver,
 	    !compare_color_256(ch->c.color, state->color)
 	   ) {
 		copy_color_256(state->color, ch->c.color);
-#ifdef CONFIG_TERMINFO
-		if (driver->opt.terminfo) {
-			add_to_string(screen, terminfo_set_bold(ch->attr & SCREEN_ATTR_BOLD));
-			add_to_string(screen, terminfo_set_foreground(ch->c.color[0]));
-			add_to_string(screen, terminfo_set_background(ch->c.color[1]));
 
-			if (ch->attr & SCREEN_ATTR_ITALIC && driver->opt.italic) {
-				state->italic = !!(ch->attr & SCREEN_ATTR_ITALIC);
-				add_to_string(screen, terminfo_set_italics(state->italic));
-			}
+		add_foreground_color(screen, driver->opt.color256_seqs, ch);
+		if (!driver->opt.transparent || ch->c.color[1] != 0) {
+			add_background_color(screen, driver->opt.color256_seqs, ch);
+		}
 
-			if (ch->attr & SCREEN_ATTR_UNDERLINE && driver->opt.underline) {
-				state->underline = !!(ch->attr & SCREEN_ATTR_UNDERLINE);
-				add_to_string(screen, terminfo_set_underline(state->underline));
-			}
-		} else
-#endif
-		{
-			add_foreground_color(screen, driver->opt.color256_seqs, ch);
-			if (!driver->opt.transparent || ch->c.color[1] != 0) {
-				add_background_color(screen, driver->opt.color256_seqs, ch);
-			}
+		if (ch->attr & SCREEN_ATTR_BOLD)
+			add_bytes_to_string(screen, "\033[1m", 4);
 
-			if (ch->attr & SCREEN_ATTR_BOLD)
-				add_bytes_to_string(screen, "\033[1m", 4);
+		if (ch->attr & SCREEN_ATTR_ITALIC && driver->opt.italic) {
+			state->italic = !!(ch->attr & SCREEN_ATTR_ITALIC);
+			add_term_string(screen, driver->opt.italic[state->italic]);
+		}
 
-			if (ch->attr & SCREEN_ATTR_ITALIC && driver->opt.italic) {
-				state->italic = !!(ch->attr & SCREEN_ATTR_ITALIC);
-				add_term_string(screen, driver->opt.italic[state->italic]);
-			}
-
-			if (ch->attr & SCREEN_ATTR_UNDERLINE && driver->opt.underline) {
-				state->underline = !!(ch->attr & SCREEN_ATTR_UNDERLINE);
-				add_term_string(screen, driver->opt.underline[state->underline]);
-			}
+		if (ch->attr & SCREEN_ATTR_UNDERLINE && driver->opt.underline) {
+			state->underline = !!(ch->attr & SCREEN_ATTR_UNDERLINE);
+			add_term_string(screen, driver->opt.underline[state->underline]);
 		}
 	}
 
@@ -1291,19 +1141,19 @@ add_char_true(struct string *screen, struct screen_driver *driver,
 {										\
 	struct terminal_screen *screen = (term_)->screen;			\
 	int y = screen->dirty_from;					\
+	int ypos = y * (term_)->width;						\
+	int prev_y = -1;							\
 	int xmax = (term_)->width - 1;						\
 	int ymax = (term_)->height - 1;						\
+	struct screen_char *current = &screen->last_image[ypos];	\
+	struct screen_char *pos = &screen->image[ypos];		\
+	struct screen_char *prev_pos = NULL; /* Warning prevention. */	\
 										\
 	int_upper_bound(&screen->dirty_to, ymax);				\
 										\
 	for (; y <= screen->dirty_to; y++) {					\
-		int ypos = y * (term_)->width;					\
-		struct screen_char *current = &screen->last_image[ypos];	\
-		struct screen_char *pos = &screen->image[ypos];			\
-		struct screen_char *start_of_line = pos;			\
 		int is_last_line = (y == ymax);					\
 		int x = 0;						\
-		int dirty = 0;						\
 										\
 		for (; x <= xmax; x++, current++, pos++) {			\
 			/*  Workaround for terminals without
@@ -1331,18 +1181,19 @@ add_char_true(struct string *screen, struct screen_driver *driver,
 				    && pos->attr == current->attr)		\
 					continue;				\
 			}							\
-			dirty = 1;						\
-			break;							\
-		}								\
 										\
-		if (!dirty) {							\
-			continue;						\
+			/* Move the cursor when @prev_pos is more than 10 chars
+			 * away. */						\
+			if (prev_y != y || prev_pos + 10 <= pos) {		\
+				add_cursor_move_to_string(image_, y + 1, x + 1);\
+				prev_pos = pos;					\
+				prev_y = y;					\
+			}							\
+				 						\
+			for (; prev_pos <= pos ; prev_pos++)			\
+				ADD_CHAR(image_, driver_, prev_pos, state_);	\
 		}								\
-		add_cursor_move_to_string(image_, y + 1, 1);			\
-		for (pos = start_of_line, x = 0; x <= xmax; x++, pos++) {	\
-			ADD_CHAR(image_, driver_, pos, state_);	\
-		}							\
-	}								\
+	}									\
 }
 
 /*! Updating of the terminal screen is done by checking what needs to
@@ -1437,12 +1288,6 @@ erase_screen(struct terminal *term)
 		want_draw();
 	}
 
-#ifdef CONFIG_TERMINFO
-	if (get_cmd_opt_bool("terminfo")) {
-		char *text = terminfo_clear_screen();
-		hard_write(term->fdout, text, strlen(text));
-	} else 
-#endif
 	hard_write(term->fdout, "\033[2J\033[1;1H", 10);
 	if (term->master) done_draw();
 }
